@@ -6,13 +6,20 @@
 --
 -- -----------------------------------------------------------------------------
 
+{-# OPTIONS -fno-warn-tabs #-}
+-- The above warning supression flag is a temporary kludge.
+-- While working on this module you are encouraged to remove it and
+-- detab the module (please do the detabbing in a separate patch). See
+--     http://hackage.haskell.org/trac/ghc/wiki/Commentary/CodingStyle#TabsvsSpaces
+-- for details
+
 module GHC (
 	-- * Initialisation
 	defaultErrorHandler,
 	defaultCleanupHandler,
 
         -- * GHC Monad
-        Ghc, GhcT, GhcMonad(..),
+        Ghc, GhcT, GhcMonad(..), HscEnv,
         runGhc, runGhcT, initGhcMonad,
         gcatch, gbracket, gfinally,
         printException,
@@ -38,7 +45,7 @@ module GHC (
 	
 	-- * Loading\/compiling the program
 	depanal,
-	load, LoadHowMuch(..),
+        load, LoadHowMuch(..), InteractiveImport(..),
 	SuccessFlag(..), succeeded, failed,
         defaultWarnErrLogger, WarnErrLogger,
 	workingDirectoryChanged,
@@ -48,13 +55,16 @@ module GHC (
         TypecheckedMod, ParsedMod,
         moduleInfo, renamedSource, typecheckedSource,
         parsedSource, coreModule,
+
+        -- ** Compiling to Core
+        CoreModule(..),
         compileToCoreModule, compileToCoreSimplified,
         compileCoreToObj,
-        getModSummary,
 
 	-- * Inspecting the module structure of the program
 	ModuleGraph, ModSummary(..), ms_mod_name, ModLocation(..),
-	getModuleGraph,
+        getModSummary,
+        getModuleGraph,
 	isLoaded,
 	topSortModuleGraph,
 
@@ -80,7 +90,7 @@ module GHC (
 	PrintUnqualified, alwaysQualify,
 
 	-- * Interactive evaluation
-	getBindings, getPrintUnqual,
+	getBindings, getInsts, getPrintUnqual,
         findModule,
         lookupModule,
 #ifdef GHCI
@@ -94,7 +104,7 @@ module GHC (
 	typeKind,
 	parseName,
 	RunResult(..),  
-	runStmt, runStmtWithLocation,
+	runStmt, runStmtWithLocation, runDecls, runDeclsWithLocation,
         parseImportDecl, SingleStep(..),
         resume,
         Resume(resumeStmt, resumeThreadId, resumeBreakInfo, resumeSpan,
@@ -145,7 +155,7 @@ module GHC (
 	TyCon, 
 	tyConTyVars, tyConDataCons, tyConArity,
 	isClassTyCon, isSynTyCon, isNewTyCon, isPrimTyCon, isFunTyCon,
-	isFamilyTyCon,
+	isFamilyTyCon, tyConClass_maybe,
 	synTyConDefn, synTyConType, synTyConResKind,
 
 	-- ** Type variables
@@ -161,19 +171,21 @@ module GHC (
 
 	-- ** Classes
 	Class, 
-	classMethods, classSCTheta, classTvsFds,
+	classMethods, classSCTheta, classTvsFds, classATs,
 	pprFundeps,
 
 	-- ** Instances
 	Instance, 
-	instanceDFunId, pprInstance, pprInstanceHdr,
+	instanceDFunId, 
+        pprInstance, pprInstanceHdr,
+        pprFamInst, pprFamInstHdr,
 
 	-- ** Types and Kinds
 	Type, splitForAllTys, funResultTy, 
 	pprParendType, pprTypeApp, 
 	Kind,
 	PredType,
-	ThetaType, pprForAll, pprThetaArrow, pprThetaArrowTy,
+	ThetaType, pprForAll, pprThetaArrowTy,
 
 	-- ** Entities
 	TyThing(..), 
@@ -188,7 +200,7 @@ module GHC (
 	compareFixity,
 
 	-- ** Source locations
-	SrcLoc(..), RealSrcLoc, pprDefnLoc,
+	SrcLoc(..), RealSrcLoc, 
         mkSrcLoc, noSrcLoc,
 	srcLocFile, srcLocLine, srcLocCol,
         SrcSpan(..), RealSrcSpan,
@@ -253,21 +265,21 @@ import Packages
 import NameSet
 import RdrName
 import qualified HsSyn -- hack as we want to reexport the whole module
-import HsSyn hiding ((<.>))
-import Type
-import Coercion		( synTyConResKind )
+import HsSyn
+import Type     hiding( typeKind )
+import Kind		( synTyConResKind )
 import TcType		hiding( typeKind )
 import Id
 import TysPrim		( alphaTyVars )
 import TyCon
 import Class
--- import FunDeps
 import DataCon
 import Name             hiding ( varName )
--- import OccName		( parenSymOcc )
+import Avail
 import InstEnv
+import FamInstEnv
 import SrcLoc
-import CoreSyn          ( CoreBind )
+import CoreSyn
 import TidyPgm
 import DriverPhases     ( Phase(..), isHaskellSrcFilename )
 import Finder
@@ -275,8 +287,7 @@ import HscTypes
 import DynFlags
 import StaticFlagParser
 import qualified StaticFlags
-import SysTools     ( initSysTools, cleanTempFiles, 
-                      cleanTempDirs )
+import SysTools
 import Annotations
 import Module
 import UniqFM
@@ -582,7 +593,8 @@ class TypecheckedMod m => DesugaredMod m where
 -- | The result of successful parsing.
 data ParsedModule =
   ParsedModule { pm_mod_summary   :: ModSummary
-               , pm_parsed_source :: ParsedSource }
+               , pm_parsed_source :: ParsedSource
+               , pm_extra_src_files :: [FilePath] }
 
 instance ParsedMod ParsedModule where
   modSummary m    = pm_mod_summary m
@@ -668,8 +680,8 @@ parseModule :: GhcMonad m => ModSummary -> m ParsedModule
 parseModule ms = do
    hsc_env <- getSession
    let hsc_env_tmp = hsc_env { hsc_dflags = ms_hspp_opts ms }
-   rdr_module <- liftIO $ hscParse hsc_env_tmp ms
-   return (ParsedModule ms rdr_module)
+   hpm <- liftIO $ hscParse hsc_env_tmp ms
+   return (ParsedModule ms (hpm_module hpm) (hpm_src_files hpm))
 
 -- | Typecheck and rename a parsed module.
 --
@@ -680,7 +692,9 @@ typecheckModule pmod = do
  hsc_env <- getSession
  let hsc_env_tmp = hsc_env { hsc_dflags = ms_hspp_opts ms }
  (tc_gbl_env, rn_info)
-       <- liftIO $ hscTypecheckRename hsc_env_tmp ms (parsedSource pmod)
+       <- liftIO $ hscTypecheckRename hsc_env_tmp ms $
+                      HsParsedModule { hpm_module = parsedSource pmod,
+                                       hpm_src_files = pm_extra_src_files pmod }
  details <- liftIO $ makeSimpleDetails hsc_env_tmp tc_gbl_env
  return $
      TypecheckedModule {
@@ -768,7 +782,7 @@ data CoreModule
       -- | Type environment for types declared in this module
       cm_types    :: !TypeEnv,
       -- | Declarations
-      cm_binds    :: [CoreBind]
+      cm_binds    :: CoreProgram
     }
 
 instance Outputable CoreModule where
@@ -866,11 +880,15 @@ compileCore simplify fn = do
         -- we just have a ModGuts.
         gutsToCoreModule :: Either (CgGuts, ModDetails) ModGuts -> CoreModule
         gutsToCoreModule (Left (cg, md))  = CoreModule {
-          cm_module = cg_module cg,    cm_types = md_types md,
+          cm_module = cg_module cg,
+          cm_types = md_types md,
           cm_binds = cg_binds cg
         }
         gutsToCoreModule (Right mg) = CoreModule {
-          cm_module  = mg_module mg,                   cm_types   = mg_types mg,
+          cm_module  = mg_module mg,
+          cm_types   = typeEnvFromEntities (bindersOfBinds (mg_binds mg))
+                                           (mg_tcs mg)
+                                           (mg_fam_insts mg),
           cm_binds   = mg_binds mg
          }
 
@@ -901,13 +919,12 @@ isLoaded m = withSession $ \hsc_env ->
 -- | Return the bindings for the current interactive session.
 getBindings :: GhcMonad m => m [TyThing]
 getBindings = withSession $ \hsc_env ->
-   -- we have to implement the shadowing behaviour of ic_tmp_ids here
-   -- (see InteractiveContext) and the quickest way is to use an OccEnv.
-   let 
-       occ_env = mkOccEnv [ (nameOccName (idName id), AnId id) 
-                          | id <- ic_tmp_ids (hsc_IC hsc_env) ]
-   in
-   return (occEnvElts occ_env)
+    return $ icInScopeTTs $ hsc_IC hsc_env
+
+-- | Return the instances for the current interactive session.
+getInsts :: GhcMonad m => m ([Instance], [FamInst])
+getInsts = withSession $ \hsc_env ->
+    return $ ic_instances (hsc_IC hsc_env)
 
 getPrintUnqual :: GhcMonad m => m PrintUnqualified
 getPrintUnqual = withSession $ \hsc_env ->
@@ -946,18 +963,11 @@ getModuleInfo mdl = withSession $ \hsc_env -> do
 
 getPackageModuleInfo :: HscEnv -> Module -> IO (Maybe ModuleInfo)
 #ifdef GHCI
-getPackageModuleInfo hsc_env mdl = do
-  mb_avails <- hscGetModuleExports hsc_env mdl
-     -- This is the only use of hscGetModuleExports.  Perhaps we could use
-     -- hscRnImportDecls instead, but that does a lot more than we need
-     -- (building instance environment, checking family instance consistency
-     -- etc.).
-  case mb_avails of
-    Nothing -> return Nothing
-    Just avails -> do
-	eps <- hscEPS hsc_env
-        iface <- lookupModuleIface hsc_env mdl
+getPackageModuleInfo hsc_env mdl 
+  = do	eps <- hscEPS hsc_env
+        iface <- hscGetModuleInterface hsc_env mdl
 	let 
+	    avails = mi_exports iface
             names  = availsToNameSet avails
 	    pte    = eps_PTE eps
 	    tys    = [ ty | name <- concatMap availNames avails,
@@ -968,7 +978,7 @@ getPackageModuleInfo hsc_env mdl = do
 			minf_exports   = names,
 			minf_rdr_env   = Just $! availsToGlobalRdrEnv (moduleName mdl) avails,
 			minf_instances = error "getModuleInfo: instances for package module unimplemented",
-                        minf_iface     = iface,
+                        minf_iface     = Just iface,
                         minf_modBreaks = emptyModBreaks  
 		}))
 #else
@@ -983,27 +993,18 @@ getHomeModuleInfo hsc_env mdl =
     Nothing  -> return Nothing
     Just hmi -> do
       let details = hm_details hmi
-      iface <- lookupModuleIface hsc_env mdl
+          iface   = hm_iface hmi
       return (Just (ModuleInfo {
 			minf_type_env  = md_types details,
 			minf_exports   = availsToNameSet (md_exports details),
 			minf_rdr_env   = mi_globals $! hm_iface hmi,
 			minf_instances = md_insts details,
-                        minf_iface     = iface
+                        minf_iface     = Just iface
 #ifdef GHCI
                        ,minf_modBreaks = getModBreaks hmi
 #endif
 			}))
 
-lookupModuleIface :: HscEnv -> Module -> IO (Maybe ModIface)
-lookupModuleIface env m = do
-    eps <- hscEPS env
-    let dflags    = hsc_dflags env
-        pkgIfaceT = eps_PIT eps
-        homePkgT  = hsc_HPT env
-        iface     = lookupIfaceByModule dflags homePkgT pkgIfaceT m
-    return iface
-
 -- | The list of top-level entities defined in a module
 modInfoTyThings :: ModuleInfo -> [TyThing]
 modInfoTyThings minf = typeEnvElts (minf_type_env minf)
