@@ -1,6 +1,13 @@
 {-# OPTIONS_GHC -fno-cse -fno-warn-orphans #-}
 -- -fno-cse is needed for GLOBAL_VAR's to behave properly
 
+{-# OPTIONS -fno-warn-tabs #-}
+-- The above warning supression flag is a temporary kludge.
+-- While working on this module you are encouraged to remove it and
+-- detab the module (please do the detabbing in a separate patch). See
+--     http://hackage.haskell.org/trac/ghc/wiki/Commentary/CodingStyle#TabsvsSp
+-- for details
+
 -----------------------------------------------------------------------------
 --
 -- Monadery code used in InteractiveUI
@@ -56,7 +63,6 @@ data GHCiState = GHCiState
 	editor         :: String,
         stop           :: String,
 	options        :: [GHCiOption],
-        prelude        :: GHC.ModuleName,
         line_number    :: !Int,         -- input line
         break_ctr      :: !Int,
         breaks         :: ![(Int, BreakLocation)],
@@ -68,23 +74,27 @@ data GHCiState = GHCiState
         -- remember is here:
         last_command   :: Maybe Command,
         cmdqueue       :: [String],
-        remembered_ctx :: [CtxtCmd],
-             -- we remember the :module commands between :loads, so that
-             -- on a :reload we can replay them.  See bugs #2049,
-             -- \#1873, #1360. Previously we tried to remember modules that
-             -- were supposed to be in the context but currently had errors,
-             -- but this was complicated.  Just replaying the :module commands
-             -- seems to be the right thing.
+
+        remembered_ctx :: [InteractiveImport],
+             -- the imports that the user has asked for, via import
+             -- declarations and :module commands.  This list is
+             -- persistent over :reloads (but any imports for modules
+             -- that are not loaded are temporarily ignored).  After a
+             -- :load, all the home-package imports are stripped from
+             -- this list.
+
+             -- See bugs #2049, #1873, #1360
+
+        transient_ctx  :: [InteractiveImport],
+             -- An import added automatically after a :load, usually of
+             -- the most recently compiled module.  May be empty if
+             -- there are no modules loaded.  This list is replaced by
+             -- :load, :reload, and :add.  In between it may be modified
+             -- by :module.
+
         ghc_e :: Bool -- True if this is 'ghc -e' (or runghc)
      }
 
-data CtxtCmd    -- In each case, the first [String] are the starred modules
-     		-- and the second are the unstarred ones
-  = SetContext [String] [String]
-  | AddModules [String] [String]
-  | RemModules [String] [String]
-  | Import     String
-
 type TickArray = Array Int [(BreakIndex,SrcSpan)]
 
 data GHCiOption 
@@ -161,6 +171,8 @@ getGHCiState :: GHCi GHCiState
 getGHCiState   = GHCi $ \r -> liftIO $ readIORef r
 setGHCiState :: GHCiState -> GHCi ()
 setGHCiState s = GHCi $ \r -> liftIO $ writeIORef r s
+modifyGHCiState :: (GHCiState -> GHCiState) -> GHCi ()
+modifyGHCiState f = GHCi $ \r -> liftIO $ readIORef r >>= writeIORef r . f
 
 liftGhc :: Ghc a -> GHCi a
 liftGhc m = GHCi $ \_ -> m
@@ -209,10 +221,6 @@ instance ExceptionMonad (InputT GHCi) where
   gblock = Haskeline.block
   gunblock = Haskeline.unblock
 
--- for convenience...
-getPrelude :: GHCi ModuleName
-getPrelude = getGHCiState >>= return . prelude
-
 getDynFlags :: GhcMonad m => m DynFlags
 getDynFlags = do
   GHC.getSessionDynFlags
@@ -246,16 +254,27 @@ printForUserPartWay doc = do
   unqual <- GHC.getPrintUnqual
   liftIO $ Outputable.printForUserPartWay stdout opt_PprUserLength unqual doc
 
-runStmt :: String -> GHC.SingleStep -> GHCi GHC.RunResult
+runStmt :: String -> GHC.SingleStep -> GHCi (Maybe GHC.RunResult)
 runStmt expr step = do
   st <- getGHCiState
   reifyGHCi $ \x ->
     withProgName (progname st) $
     withArgs (args st) $
       reflectGHCi x $ do
-        GHC.handleSourceError (\e -> do GHC.printException e
-                                        return GHC.RunFailed) $ do
-          GHC.runStmtWithLocation (progname st) (line_number st) expr step 
+        GHC.handleSourceError (\e -> do GHC.printException e; 
+                                        return Nothing) $ do
+          r <- GHC.runStmtWithLocation (progname st) (line_number st) expr step
+          return (Just r)
+
+runDecls :: String -> GHCi [GHC.Name]
+runDecls decls = do
+  st <- getGHCiState
+  reifyGHCi $ \x ->
+    withProgName (progname st) $
+    withArgs (args st) $
+      reflectGHCi x $ do
+        GHC.handleSourceError (\e -> do GHC.printException e; return []) $ do
+          GHC.runDeclsWithLocation (progname st) (line_number st) decls
 
 resume :: (SrcSpan -> Bool) -> GHC.SingleStep -> GHCi GHC.RunResult
 resume canLogSpan step = do
@@ -361,4 +380,4 @@ turnOffBuffering
 getHandle :: IORef (Ptr ()) -> IO Handle
 getHandle ref = do
   (Ptr addr) <- readIORef ref
-  case addrToHValue# addr of (# hval #) -> return (unsafeCoerce# hval)
+  case addrToAny# addr of (# hval #) -> return (unsafeCoerce# hval)
