@@ -20,8 +20,9 @@
 //
 StgTSO *createThread (Capability *cap, nat stack_size);
 
-Capability *scheduleWaitThread (StgTSO *tso, /*out*/HaskellObj* ret,
-				Capability *cap);
+void scheduleWaitThread (/* in    */ StgTSO *tso,
+                         /* out   */ HaskellObj* ret,
+                         /* inout */ Capability **cap);
 
 StgTSO *createGenThread       (Capability *cap, nat stack_size,  
 			       StgClosure *closure);
@@ -56,4 +57,14 @@ extern unsigned int n_capabilities;
 extern Capability MainCapability;
 #endif
 
+//
+// Change the number of capabilities (only supports increasing the
+// current value at the moment).
+//
+#if defined(THREADED_RTS)
+extern void setNumCapabilities (nat new);
+#else
+extern void setNumCapabilities (nat new) GNU_ATTRIBUTE(__noreturn__);
+#endif
+
 #endif /* RTS_THREADS_H */
