@@ -1,6 +1,6 @@
 /* -----------------------------------------------------------------------------
  *
- * (c) The GHC Team, 1998-2009
+ * (c) The GHC Team, 1998-2011
  *
  * Registers used in STG code.  Might or might not correspond to
  * actual machine registers.
@@ -38,26 +38,18 @@
  * we want to know about the register mapping on the TARGET platform.
  */
 #ifdef COMPILING_GHC
-#define alpha_REGS    alpha_TARGET_ARCH
-#define hppa1_1_REGS  hppa1_1_TARGET_ARCH
 #define i386_REGS     i386_TARGET_ARCH
 #define x86_64_REGS   x86_64_TARGET_ARCH
-#define m68k_REGS     m68k_TARGET_ARCH
-#define mips_REGS     (mipsel_TARGET_ARCH || mipseb_TARGET_ARCH)
 #define powerpc_REGS  (powerpc_TARGET_ARCH || powerpc64_TARGET_ARCH || rs6000_TARGET_ARCH)
-#define ia64_REGS     ia64_TARGET_ARCH
 #define sparc_REGS    sparc_TARGET_ARCH
+#define arm_REGS      arm_TARGET_ARCH
 #define darwin_REGS   darwin_TARGET_OS
 #else
-#define alpha_REGS    alpha_HOST_ARCH
-#define hppa1_1_REGS  hppa1_1_HOST_ARCH
 #define i386_REGS     i386_HOST_ARCH
 #define x86_64_REGS   x86_64_HOST_ARCH
-#define m68k_REGS     m68k_HOST_ARCH
-#define mips_REGS     (mipsel_HOST_ARCH || mipseb_HOST_ARCH)
 #define powerpc_REGS  (powerpc_HOST_ARCH || powerpc64_HOST_ARCH || rs6000_HOST_ARCH)
-#define ia64_REGS     ia64_HOST_ARCH
 #define sparc_REGS    sparc_HOST_ARCH
+#define arm_REGS      arm_HOST_ARCH
 #define darwin_REGS   darwin_HOST_OS
 #endif
 
@@ -74,139 +66,6 @@
    this code path is completely untested! -- EZY
    -------------------------------------------------------------------------- */
 
-/* -----------------------------------------------------------------------------
-   The DEC Alpha register mapping
-
-   Alpha registers
-   \tr{$9}--\tr{$14} are our ``prize'' callee-save registers.  
-   \tr{$15} is the frame pointer, and \tr{$16}--\tr{$21} are argument
-   registers.  (These are off-limits.)  We can steal some of the \tr{$22}-and-up 
-   caller-save registers provided we do the appropriate save/restore stuff.
-   
-   \tr{$f2}--\tr{$f9} are some callee-save floating-point registers.
-   
-   We cannot use \tr{$23} (aka t9), \tr{$24} (aka t10), \tr{$25} (aka
-   t11), \tr{$27} (aka pv), or \tr{$28} (aka at), because they are
-   occasionally required by the assembler to handle non-primitive
-   instructions (e.g. ldb, remq).  Sigh!
-   
-   Cheat sheet for GDB:
-   
-   GDB	here	Main map
-   ===	====	========
-   s5	$14	R1
-   t1	$2	R2
-   t2	$3	R3
-   t3	$4	R4
-   t4	$5	R5
-   t5	$6	R6
-   t6	$7	R7
-   t7	$8	R8
-   s0	$9	Sp
-   s2	$11	SpLim
-   s3	$12	Hp   
-   t8	$22	NCG_reserved
-   t12	$27	NCG_reserved
-   -------------------------------------------------------------------------- */
-
-#if alpha_REGS
-# define REG(x) __asm__("$" #x)
-
-#  define CALLER_SAVES_R2
-#  define CALLER_SAVES_R3
-#  define CALLER_SAVES_R4
-#  define CALLER_SAVES_R5
-#  define CALLER_SAVES_R6
-#  define CALLER_SAVES_R7
-#  define CALLER_SAVES_R8
-  
-#  define CALLER_SAVES_USER
-  
-#  define REG_R1	14
-#  define REG_R2    	2
-#  define REG_R3    	3
-#  define REG_R4    	4
-#  define REG_R5    	5
-#  define REG_R6    	6
-#  define REG_R7    	7
-#  define REG_R8    	8
-  
-#  define REG_F1	f2
-#  define REG_F2	f3
-#  define REG_F3	f4
-#  define REG_F4	f5
-  
-#  define REG_D1	f6
-#  define REG_D2	f7
-  
-#  define REG_Sp    	9
-#  define REG_SpLim     11
-
-#  define REG_Hp	12
-  
-#  define NCG_Reserved_I1 22
-#  define NCG_Reserved_I2 27
-#  define NCG_Reserved_F1 f29
-#  define NCG_Reserved_F2 f30
-
-#endif /* alpha_REGS */
-
-/* -----------------------------------------------------------------------------
-   The HP-PA register mapping
-
-   We cater for HP-PA 1.1.
-   
-   \tr{%r0}--\tr{%r1} are special.
-   \tr{%r2} is the return pointer.
-   \tr{%r3} is the frame pointer.
-   \tr{%r4}--\tr{%r18} are callee-save registers.
-   \tr{%r19} is a linkage table register for HPUX 8.0 shared libraries.
-   \tr{%r20}--\tr{%r22} are caller-save registers.
-   \tr{%r23}--\tr{%r26} are parameter registers.
-   \tr{%r27} is a global data pointer.
-   \tr{%r28}--\tr{%r29} are temporaries.
-   \tr{%r30} is the stack pointer.
-   \tr{%r31} is a temporary.
-   
-   \tr{%fr12}--\tr{%fr15} are some callee-save floating-point registers.
-   \tr{%fr8}--\tr{%fr11} are some available caller-save fl-pt registers.
-   -------------------------------------------------------------------------- */
-
-#if hppa1_1_REGS
-
-#define REG(x) __asm__("%" #x)
-
-#define REG_R1		r11
-#define REG_R2    	r12
-#define REG_R3    	r13
-#define REG_R4    	r14
-#define REG_R5    	r15
-#define REG_R6    	r16
-#define REG_R7    	r17
-#define REG_R8    	r18
-
-#define REG_F1		fr12
-#define REG_F2		fr12R
-#define REG_F3		fr13
-#define REG_F4		fr13R
-
-#define REG_D1		fr20	/* L & R */
-#define REG_D2		fr21	/* L & R */
-
-#define REG_Sp    	r4
-#define REG_SpLim    	r6
-
-#define REG_Hp	    	r7
-
-#define NCG_Reserved_I1 r28
-#define NCG_Reserved_I2	r29
-#define NCG_Reserved_F1	fr8
-#define NCG_Reserved_F2	fr8R
-#define NCG_Reserved_D1	fr10
-#define NCG_Reserved_D2	fr11
-
-#endif /* hppa */
-
 /* -----------------------------------------------------------------------------
    The x86 register mapping
 
@@ -225,7 +84,6 @@
    Leaving SpLim out of the picture.
    -------------------------------------------------------------------------- */
 
-
 #if i386_REGS
 
 #define REG(x) __asm__("%" #x)
@@ -326,116 +184,6 @@
 
 #endif /* x86_64 */
 
-/* -----------------------------------------------------------------------------
-   The Motorola 680x0 register mapping
-
-   A Sun3 (mc680x0) has eight address registers, \tr{a0} to \tr{a7}, and
-   eight data registers, \tr{d0} to \tr{d7}.  Address operations have to
-   be done through address registers; data registers are used for
-   comparison values and data.
-   
-   Here's the register-usage picture for m68k boxes with GCC.
-
-   \begin{tabular}{ll}
-   a0 & used directly by GCC \\
-   a1 & used directly by GCC \\
-   \\
-   a2..a5 & callee-saved: available for STG registers \\
-   & (a5 may be special, ``global'' register for PIC?) \\
-   \\
-   a6 & C-stack frame pointer \\
-   a7 & C-stack pointer \\
-   \\
-   d0 & used directly by GCC \\
-   d1 & used directly by GCC \\
-   d2 & really needed for local optimisation by GCC \\
-   \\
-   d3..d7 & callee-saved: available for STG registers
-   \\
-   fp0 & call-clobbered \\
-   fp1 & call-clobbered \\
-   fp2..fp7 & callee-saved: available for STG registers
-   \end{tabular}
-   -------------------------------------------------------------------------- */
-
-#if m68k_REGS
-
-#define REG(x) __asm__(#x)
-
-#define REG_Base	a2
-
-#define REG_Sp		a3
-#define REG_SpLim	d3
-
-#define REG_Hp	    	d4
-
-#define REG_R1    	a5
-#define REG_R2    	d6
-#define MAX_REAL_VANILLA_REG 2
-
-#define REG_Ret		d7
-
-#define REG_F1		fp2
-#define REG_F2		fp3
-#define REG_F3		fp4
-#define REG_F4		fp5
-
-#define REG_D1		fp6
-#define REG_D2		fp7
-
-#endif /* m68k */
-
-/* -----------------------------------------------------------------------------
-   The DECstation (MIPS) register mapping
-
-   Here's at least some simple stuff about registers on a MIPS.
-   
-   \tr{s0}--\tr{s7} are callee-save integer registers; they are our
-   ``prize'' stolen registers.  There is also a wad of callee-save
-   floating-point registers, \tr{$f20}--\tr{$f31}; we'll use some of
-   those.
-   
-   \tr{t0}--\tr{t9} are caller-save (``temporary?'') integer registers.
-   We can steal some, but we might have to save/restore around ccalls.
-   -------------------------------------------------------------------------- */
-
-#if mips_REGS
-
-#define REG(x) __asm__("$" #x)
-
-#define CALLER_SAVES_R5
-#define CALLER_SAVES_R6
-#define CALLER_SAVES_R7
-#define CALLER_SAVES_R8
-
-#define CALLER_SAVES_USER
-
-#define REG_R1		16
-#define REG_R2    	17
-#define REG_R3    	18
-#define REG_R4    	19
-#define REG_R5    	12
-#define REG_R6    	13
-#define REG_R7    	14
-#define REG_R8    	15
-
-#define REG_F1		f20
-#define REG_F2		f22
-#define REG_F3		f24
-#define REG_F4		f26
-
-#define REG_D1		f28
-#define REG_D2		f30
-
-#define REG_Sp    	20
-#define REG_SpLim    	21
-
-#define REG_Hp	    	22
-
-#define REG_Base	30
-
-#endif /* mipse[lb] */
-
 /* -----------------------------------------------------------------------------
    The PowerPC register mapping
 
@@ -510,46 +258,6 @@
 
 #endif /* powerpc */
 
-/* -----------------------------------------------------------------------------
-   The IA64 register mapping
-
-   We place the general registers in the locals area of the register stack,
-   so that the call mechanism takes care of saving them for us.  We reserve
-   the first 16 for gcc's use - since gcc uses the highest used register to
-   determine the register stack frame size, this gives us a constant size
-   register stack frame.
-   
-   \tr{f16-f32} are the callee-saved floating point registers.
-   -------------------------------------------------------------------------- */
-
-#if ia64_REGS
-
-#define REG(x) __asm__(#x)
-
-#define REG_R1		loc16
-#define REG_R2		loc17
-#define REG_R3		loc18
-#define REG_R4		loc19
-#define REG_R5		loc20
-#define REG_R6		loc21
-#define REG_R7		loc22
-#define REG_R8		loc23
-
-#define REG_F1		f16
-#define REG_F2		f17
-#define REG_F3		f18
-#define REG_F4		f19
-
-#define REG_D1		f20
-#define REG_D2		f21
-
-#define REG_Sp		loc24
-#define REG_SpLim	loc26
-
-#define REG_Hp		loc27
-
-#endif /* ia64 */
-
 /* -----------------------------------------------------------------------------
    The Sun SPARC register mapping
 
@@ -695,6 +403,69 @@
 
 #endif /* sparc */
 
+/* -----------------------------------------------------------------------------
+   The ARM EABI register mapping
+
+   Here we consider ARM mode (i.e. 32bit isns)
+   and also CPU with full VFPv3 implementation
+
+   ARM registers (see Chapter 5.1 in ARM IHI 0042D)
+
+   r15  PC         The Program Counter.
+   r14  LR         The Link Register.
+   r13  SP         The Stack Pointer.
+   r12  IP         The Intra-Procedure-call scratch register.
+   r11  v8         Variable-register 8.
+   r10  v7         Variable-register 7.
+   r9   v6/SB/TR   Platform register. The meaning of this register is
+                   defined by the platform standard.
+   r8   v5         Variable-register 5.
+   r7   v4         Variable register 4.
+   r6   v3         Variable register 3.
+   r5   v2         Variable register 2.
+   r4   v1         Variable register 1.
+   r3   a4         Argument / scratch register 4.
+   r2   a3         Argument / scratch register 3.
+   r1   a2         Argument / result / scratch register 2.
+   r0   a1         Argument / result / scratch register 1.
+
+   VFPv2/VFPv3/NEON registers
+   s0-s15/d0-d7/q0-q3    Argument / result/ scratch registers
+   s16-s31/d8-d15/q4-q7  callee-saved registers (must be preserved across
+                         subrutine calls)
+
+   VFPv3/NEON registers (added to the VFPv2 registers set)
+   d16-d31/q8-q15        Argument / result/ scratch registers
+   ----------------------------------------------------------------------------- */
+
+
+#if arm_REGS
+
+#define REG(x) __asm__(#x)
+
+#define REG_Base        r4
+#define REG_Sp          r5
+#define REG_Hp          r6
+#define REG_R1          r7
+#define REG_R2          r8
+#define REG_R3          r9
+#define REG_R4          r10
+#define REG_SpLim       r11
+
+#if !defined(arm_HOST_ARCH_PRE_ARMv6)
+/* d8 */
+#define REG_F1    s16
+#define REG_F2    s17
+/* d9 */
+#define REG_F3    s18
+#define REG_F4    s19
+
+#define REG_D1    d10
+#define REG_D2    d11
+#endif
+
+#endif /* arm */
+
 #endif /* NO_REGS */
 
 /* -----------------------------------------------------------------------------
@@ -713,7 +484,11 @@
  */
 
 #ifndef MAX_REAL_VANILLA_REG
-#  if   defined(REG_R8)
+#  if   defined(REG_R10)
+#  define MAX_REAL_VANILLA_REG 10
+#  elif   defined(REG_R9)
+#  define MAX_REAL_VANILLA_REG 9
+#  elif   defined(REG_R8)
 #  define MAX_REAL_VANILLA_REG 8
 #  elif defined(REG_R7)
 #  define MAX_REAL_VANILLA_REG 7
