@@ -1,3 +1,9 @@
+{-# LANGUAGE Trustworthy #-}
+{-# LANGUAGE CPP #-}
+#ifdef __GLASGOW_HASKELL__
+{-# LANGUAGE DeriveDataTypeable, StandaloneDeriving #-}
+#endif
+
 -----------------------------------------------------------------------------
 -- |
 -- Module      :  Control.Concurrent.SampleVar
@@ -34,6 +40,10 @@ import Control.Exception ( mask_ )
 
 import Data.Functor ( (<$>) )
 
+import Data.Typeable
+
+#include "Typeable.h"
+
 -- |
 -- Sample variables are slightly different from a normal 'MVar':
 -- 
@@ -58,6 +68,8 @@ newtype SampleVar a = SampleVar ( MVar ( Int    -- 1  == full
                                 )
     deriving (Eq)
 
+INSTANCE_TYPEABLE1(SampleVar,sampleVarTc,"SampleVar")
+
 -- |Build a new, empty, 'SampleVar'
 newEmptySampleVar :: IO (SampleVar a)
 newEmptySampleVar = do
