@@ -39,7 +39,6 @@ import GHC.IO.FD
 import GHC.IO.Buffer
 import qualified GHC.IO.BufferedIO as Buffered
 import GHC.IO.Exception
-import GHC.IO.Encoding.Failure (surrogatifyRoundtripCharacter, desurrogatifyRoundtripCharacter)
 import GHC.Exception
 import GHC.IO.Handle.Types
 import GHC.IO.Handle.Internals
@@ -273,6 +272,9 @@ unpack !buf !r !w acc0
         unpackRB acc !i
          | i < r  = return acc
          | otherwise = do
+              -- Here, we are rather careful to only put an *evaluated* character
+              -- in the output string. Due to pointer tagging, this allows the consumer
+              -- to avoid ping-ponging between the actual consumer code and the thunk code
 #ifdef CHARBUF_UTF16
               -- reverse-order decoding of UTF-16
               c2 <- peekElemOff pbuf i
@@ -281,10 +283,11 @@ unpack !buf !r !w acc0
                  else do c1 <- peekElemOff pbuf (i-1)
                          let c = (fromIntegral c1 - 0xd800) * 0x400 +
                                  (fromIntegral c2 - 0xdc00) + 0x10000
-                         unpackRB (desurrogatifyRoundtripCharacter (unsafeChr c) : acc) (i-2)
+                         case desurrogatifyRoundtripCharacter (unsafeChr c) of
+                           { C# c# -> unpackRB (C# c# : acc) (i-2) }
 #else
               c <- peekElemOff pbuf i
-              unpackRB (desurrogatifyRoundtripCharacter c:acc) (i-1)
+              unpackRB (c : acc) (i-1)
 #endif
      in
      unpackRB acc0 (w-1)
@@ -307,7 +310,7 @@ unpack_nl !buf !r !w acc0
                             then unpackRB ('\n':acc) (i-2)
                             else unpackRB ('\n':acc) (i-1)
                  else do
-                         unpackRB (desurrogatifyRoundtripCharacter c:acc) (i-1)
+                         unpackRB (c : acc) (i-1)
      in do
      c <- peekElemOff pbuf (w-1)
      if (c == '\r')
@@ -321,6 +324,24 @@ unpack_nl !buf !r !w acc0
                 str <- unpackRB acc0 (w-1)
                 return (str, w)
 
+-- Note [#5536]
+--
+-- We originally had
+--
+--    let c' = desurrogatifyRoundtripCharacter c in
+--    c' `seq` unpackRB (c':acc) (i-1)
+--
+-- but this resulted in Core like
+--
+--    case (case x <# y of True -> C# e1; False -> C# e2) of c
+--      C# _ -> unpackRB (c:acc) (i-1)
+--
+-- which compiles into a continuation for the outer case, with each
+-- branch of the inner case building a C# and then jumping to the
+-- continuation.  We'd rather not have this extra jump, which makes
+-- quite a difference to performance (see #5536) It turns out that
+-- matching on the C# directly causes GHC to do the case-of-case,
+-- giving much straighter code.
 
 -- -----------------------------------------------------------------------------
 -- hGetContents
@@ -587,7 +608,7 @@ writeBlocks hdl line_buffered add_nl nl
            else do
                shoveString n' cs rest
      | otherwise = do
-        n' <- writeCharBuf raw n (surrogatifyRoundtripCharacter c)
+        n' <- writeCharBuf raw n c
         shoveString n' cs rest
   in
   shoveString 0 s (if add_nl then "\n" else "")
@@ -985,3 +1006,4 @@ illegalBufferSize handle fn sz =
                             InvalidArgument  fn
                             ("illegal buffer size " ++ showsPrec 9 sz [])
                             Nothing Nothing)
+
