@@ -1,5 +1,9 @@
 {-# OPTIONS_GHC -fno-warn-unused-imports #-}
 -- apparent bug in GHC, reports a bogus warning for the Prelude import below
+#if __GLASGOW_HASKELL__ >= 701
+{-# LANGUAGE Safe #-}
+#endif
+
 module System.IO.Error (
       -- * I\/O errors
     IOError,                    -- = IOException
