@@ -1,4 +1,7 @@
 {-# LANGUAGE GADTs, RankNTypes, ScopedTypeVariables, TypeFamilies  #-}
+#if __GLASGOW_HASKELL__ >= 701
+{-# LANGUAGE Trustworthy #-}
+#endif
 
 -- | Utilities for clients of Hoopl, not used internally.
 
