@@ -1,3 +1,8 @@
+{-# LANGUAGE CPP #-}
+#if __GLASGOW_HASKELL__ >= 701
+{-# LANGUAGE Trustworthy #-}
+#endif
+
 -----------------------------------------------------------------------------
 -- |
 -- Module      :  System.Cmd
@@ -39,3 +44,4 @@ translate str = '"' : snd (foldr escape (True,"\"") str)
         escape c    (b,     str) = (False, c : str)
 
 #endif
+
