@@ -1,8 +1,9 @@
 {-# LANGUAGE CPP, ForeignFunctionInterface #-}
 #if __GLASGOW_HASKELL__ >= 701
--- not available prior to 7.1
+{-# LANGUAGE Trustworthy #-}
 {-# LANGUAGE InterruptibleFFI #-}
 #endif
+
 -----------------------------------------------------------------------------
 -- |
 -- Module      :  System.Process
@@ -18,36 +19,36 @@
 -----------------------------------------------------------------------------
 
 -- ToDo:
---	* Flag to control whether exiting the parent also kills the child.
+--      * Flag to control whether exiting the parent also kills the child.
 
 {- NOTES on createPipe:
  
    createPipe is no longer exported, because of the following problems:
 
-	- it wasn't used to implement runInteractiveProcess on Unix, because
-	  the file descriptors for the unused ends of the pipe need to be closed
-	  in the child process.
+        - it wasn't used to implement runInteractiveProcess on Unix, because
+          the file descriptors for the unused ends of the pipe need to be closed
+          in the child process.
 
         - on Windows, a special version of createPipe is needed that sets
-	  the inheritance flags correctly on the ends of the pipe (see
-	  mkAnonPipe below).
+          the inheritance flags correctly on the ends of the pipe (see
+          mkAnonPipe below).
 -}
 
 module System.Process (
 #ifndef __HUGS__
-	-- * Running sub-processes
+        -- * Running sub-processes
         createProcess,
         shell, proc,
         CreateProcess(..),
         CmdSpec(..),
         StdStream(..),
-	ProcessHandle,
+        ProcessHandle,
 
         -- ** Specific variants of createProcess
-	runCommand,
-	runProcess,
-	runInteractiveCommand,
-	runInteractiveProcess,
+        runCommand,
+        runProcess,
+        runInteractiveCommand,
+        runInteractiveProcess,
         readProcess,
         readProcessWithExitCode,
 #endif
@@ -56,11 +57,11 @@ module System.Process (
         showCommandForUser,
 
 #ifndef __HUGS__
-	-- * Process completion
-	waitForProcess,
-	getProcessExitCode,
-	terminateProcess,
-	interruptProcessGroupOf,
+        -- * Process completion
+        waitForProcess,
+        getProcessExitCode,
+        terminateProcess,
+        interruptProcessGroupOf,
 #endif
  ) where
 
@@ -70,6 +71,12 @@ import Prelude hiding (mapM)
 import System.Process.Internals
 
 import System.IO.Error
+#if !defined(mingw32_HOST_OS)
+import System.Posix.Types
+#if MIN_VERSION_unix(2,5,0)
+import System.Posix.Process (getProcessGroupIDOf)
+#endif
+#endif
 import qualified Control.Exception as C
 import Control.Concurrent
 import Control.Monad
@@ -78,13 +85,13 @@ import Foreign.C
 import System.IO
 import Data.Maybe
 #endif
-import System.Exit	( ExitCode(..) )
+import System.Exit      ( ExitCode(..) )
 
 #ifdef __GLASGOW_HASKELL__
 #if __GLASGOW_HASKELL__ >= 611
-import GHC.IO.Exception	( ioException, IOErrorType(..) )
+import GHC.IO.Exception ( ioException, IOErrorType(..) )
 #else
-import GHC.IOBase	( ioException, IOErrorType(..) )
+import GHC.IOBase       ( ioException, IOErrorType(..) )
 #endif
 #if defined(mingw32_HOST_OS)
 import System.Win32.Process (getProcessId)
@@ -132,13 +139,13 @@ runCommand string = do
      'runProcess'.
 -}
 runProcess
-  :: FilePath			-- ^ Filename of the executable
-  -> [String]			-- ^ Arguments to pass to the executable
-  -> Maybe FilePath		-- ^ Optional path to the working directory
-  -> Maybe [(String,String)]	-- ^ Optional environment (otherwise inherit)
-  -> Maybe Handle		-- ^ Handle to use for @stdin@ (Nothing => use existing @stdin@)
-  -> Maybe Handle		-- ^ Handle to use for @stdout@ (Nothing => use existing @stdout@)
-  -> Maybe Handle		-- ^ Handle to use for @stderr@ (Nothing => use existing @stderr@)
+  :: FilePath                   -- ^ Filename of the executable
+  -> [String]                   -- ^ Arguments to pass to the executable
+  -> Maybe FilePath             -- ^ Optional path to the working directory
+  -> Maybe [(String,String)]    -- ^ Optional environment (otherwise inherit)
+  -> Maybe Handle               -- ^ Handle to use for @stdin@ (Nothing => use existing @stdin@)
+  -> Maybe Handle               -- ^ Handle to use for @stdout@ (Nothing => use existing @stdout@)
+  -> Maybe Handle               -- ^ Handle to use for @stderr@ (Nothing => use existing @stderr@)
   -> IO ProcessHandle
 
 runProcess cmd args mb_cwd mb_env mb_stdin mb_stdout mb_stderr = do
@@ -149,7 +156,7 @@ runProcess cmd args mb_cwd mb_env mb_stdin mb_stdout mb_stderr = do
                           std_in  = mbToStd mb_stdin,
                           std_out = mbToStd mb_stdout,
                           std_err = mbToStd mb_stderr }
-	  Nothing Nothing
+          Nothing Nothing
   maybeClose mb_stdin
   maybeClose mb_stdout
   maybeClose mb_stderr
@@ -276,10 +283,10 @@ runInteractiveCommand string =
     in text mode then use 'hSetBinaryMode'.
 -}
 runInteractiveProcess
-  :: FilePath			-- ^ Filename of the executable
-  -> [String]			-- ^ Arguments to pass to the executable
-  -> Maybe FilePath		-- ^ Optional path to the working directory
-  -> Maybe [(String,String)]	-- ^ Optional environment (otherwise inherit)
+  :: FilePath                   -- ^ Filename of the executable
+  -> [String]                   -- ^ Arguments to pass to the executable
+  -> Maybe FilePath             -- ^ Optional path to the working directory
+  -> Maybe [(String,String)]    -- ^ Optional environment (otherwise inherit)
   -> IO (Handle,Handle,Handle,ProcessHandle)
 
 runInteractiveProcess cmd args mb_cwd mb_env = do
@@ -316,9 +323,9 @@ waitForProcess ph = do
   case p_ of
     ClosedHandle e -> return e
     OpenHandle h  -> do
-	-- don't hold the MVar while we call c_waitForProcess...
-	-- (XXX but there's a small race window here during which another
-	-- thread could close the handle or call waitForProcess)
+        -- don't hold the MVar while we call c_waitForProcess...
+        -- (XXX but there's a small race window here during which another
+        -- thread could close the handle or call waitForProcess)
         alloca $ \pret -> do
           throwErrnoIfMinus1Retry_ "waitForProcess" (c_waitForProcess h pret)
           withProcessHandle ph $ \p_' ->
@@ -341,6 +348,9 @@ waitForProcess ph = do
 -- Output is returned strictly, so this is not suitable for
 -- interactive applications.
 --
+-- This function throws an 'IOError' if the process 'ExitCode' is
+-- anything other than 'ExitSuccess'.
+--
 -- Users of this function should compile with @-threaded@ if they
 -- want other Haskell threads to keep running while waiting on
 -- the result of readProcess.
@@ -484,7 +494,7 @@ syncProcess fun c = do
   old_int  <- installHandler sigINT  Ignore Nothing
   old_quit <- installHandler sigQUIT Ignore Nothing
   (_,_,_,p) <- runGenProcess_ fun c
-		(Just defaultSignal) (Just defaultSignal)
+                (Just defaultSignal) (Just defaultSignal)
   r <- waitForProcess p
   _ <- installHandler sigINT  old_int Nothing
   _ <- installHandler sigQUIT old_quit Nothing
@@ -546,9 +556,9 @@ terminateProcess ph = do
       ClosedHandle _ -> return p_
       OpenHandle h -> do
         throwErrnoIfMinus1Retry_ "terminateProcess" $ c_terminateProcess h
-	return p_
-	-- does not close the handle, we might want to try terminating it
-	-- again, or get its exit code.
+        return p_
+        -- does not close the handle, we might want to try terminating it
+        -- again, or get its exit code.
 
 -- ----------------------------------------------------------------------------
 -- interruptProcessGroupOf
@@ -561,7 +571,7 @@ terminateProcess ph = do
 -- processes created using 'createProcess' and setting the 'create_group' flag
 
 interruptProcessGroupOf
-    :: ProcessHandle    -- ^ Lead process in the process group
+    :: ProcessHandle    -- ^ A process in the process group
     -> IO ()
 interruptProcessGroupOf ph = do
 #if mingw32_HOST_OS
@@ -577,7 +587,13 @@ interruptProcessGroupOf ph = do
         case p_ of
             ClosedHandle _ -> return p_
             OpenHandle h -> do
+#if MIN_VERSION_unix(2,5,0)
+                -- getProcessGroupIDOf was added in unix-2.5.0.0
+                pgid <- getProcessGroupIDOf h
+                signalProcessGroup sigINT pgid
+#else
                 signalProcessGroup sigINT h
+#endif
                 return p_
 #endif
 
@@ -595,31 +611,31 @@ getProcessExitCode ph = do
     case p_ of
       ClosedHandle e -> return (p_, Just e)
       OpenHandle h ->
-	alloca $ \pExitCode -> do
+        alloca $ \pExitCode -> do
             res <- throwErrnoIfMinus1Retry "getProcessExitCode" $
-	        	c_getProcessExitCode h pExitCode
-	    code <- peek pExitCode
-	    if res == 0
-	      then return (p_, Nothing)
-	      else do
-		   closePHANDLE h
-		   let e  | code == 0 = ExitSuccess
-			  | otherwise = ExitFailure (fromIntegral code)
-		   return (ClosedHandle e, Just e)
+                        c_getProcessExitCode h pExitCode
+            code <- peek pExitCode
+            if res == 0
+              then return (p_, Nothing)
+              else do
+                   closePHANDLE h
+                   let e  | code == 0 = ExitSuccess
+                          | otherwise = ExitFailure (fromIntegral code)
+                   return (ClosedHandle e, Just e)
 
 -- ----------------------------------------------------------------------------
 -- Interface to C bits
 
 foreign import ccall unsafe "terminateProcess"
   c_terminateProcess
-	:: PHANDLE
-	-> IO CInt
+        :: PHANDLE
+        -> IO CInt
 
 foreign import ccall unsafe "getProcessExitCode"
   c_getProcessExitCode
-	:: PHANDLE
-	-> Ptr CInt
-	-> IO CInt
+        :: PHANDLE
+        -> Ptr CInt
+        -> IO CInt
 
 #if __GLASGOW_HASKELL__ < 701
 -- not available prior to 7.1
@@ -628,7 +644,8 @@ foreign import ccall unsafe "getProcessExitCode"
 
 foreign import ccall interruptible "waitForProcess" -- NB. safe - can block
   c_waitForProcess
-	:: PHANDLE
+        :: PHANDLE
         -> Ptr CInt
-	-> IO CInt
+        -> IO CInt
 #endif /* !__HUGS__ */
+
