@@ -6,18 +6,20 @@ module Language.Haskell.TH.Ppr where
     -- be "public" functions.  The main module TH
     -- re-exports them all.
 
-import Text.PrettyPrint.HughesPJ (render)
+import Text.PrettyPrint (render)
 import Language.Haskell.TH.PprLib
 import Language.Haskell.TH.Syntax
 import Data.Char ( toLower )
+import GHC.Show  ( showMultiLineString )
 
 nestDepth :: Int
 nestDepth = 4
 
 type Precedence = Int
-appPrec, opPrec, noPrec :: Precedence
-appPrec = 2    -- Argument of a function application
-opPrec  = 1    -- Argument of an infix operator
+appPrec, unopPrec, opPrec, noPrec :: Precedence
+appPrec = 3    -- Argument of a function application
+opPrec  = 2    -- Argument of an infix operator
+unopPrec = 1   -- Argument of an unresolved infix operator
 noPrec  = 0    -- Others
 
 parensIf :: Bool -> Doc -> Doc
@@ -43,8 +45,9 @@ instance Ppr Name where
 
 ------------------------------
 instance Ppr Info where
-    ppr (ClassI d is) = ppr d $$ vcat (map ppr is)
-    ppr (TyConI d)    = ppr d
+    ppr (TyConI d)     = ppr d
+    ppr (ClassI d is)  = ppr d $$ vcat (map ppr is)
+    ppr (FamilyI d is) = ppr d $$ vcat (map ppr is)
     ppr (PrimTyConI name arity is_unlifted) 
       = text "Primitive"
 	<+> (if is_unlifted then text "unlifted" else empty)
@@ -62,15 +65,6 @@ instance Ppr Info where
       = vcat [ppr_sig v ty, pprFixity v fix, 
               case mb_d of { Nothing -> empty; Just d -> ppr d }]
 
-instance Ppr ClassInstance where
-  ppr (ClassInstance { ci_dfun = _dfun,
-      		       ci_tvs  = _tvs,
-      		       ci_cxt  = cxt,
-      		       ci_cls  = cls,
-                       ci_tys = tys })
-    = text "instance" <+> pprCxt cxt 
-      <+> ppr cls <+> sep (map pprParendType tys)
-
 ppr_sig :: Name -> Type -> Doc
 ppr_sig v ty = ppr v <+> text "::" <+> ppr ty
 
@@ -97,6 +91,11 @@ pprExp _ (ConE c)     = pprName' Applied c
 pprExp i (LitE l)     = pprLit i l
 pprExp i (AppE e1 e2) = parensIf (i >= appPrec) $ pprExp opPrec e1
                                               <+> pprExp appPrec e2
+pprExp _ (ParensE e)  = parens (pprExp noPrec e)
+pprExp i (UInfixE e1 op e2)
+ = parensIf (i > unopPrec) $ pprExp unopPrec e1
+                         <+> pprInfixExp op
+                         <+> pprExp unopPrec e2
 pprExp i (InfixE (Just e1) op (Just e2))
  = parensIf (i >= opPrec) $ pprExp opPrec e1
                         <+> pprInfixExp op
@@ -107,6 +106,7 @@ pprExp _ (InfixE me1 op me2) = parens $ pprMaybeExp noPrec me1
 pprExp i (LamE ps e) = parensIf (i > noPrec) $ char '\\' <> hsep (map (pprPat appPrec) ps)
                                            <+> text "->" <+> ppr e
 pprExp _ (TupE es) = parens $ sep $ punctuate comma $ map ppr es
+pprExp _ (UnboxedTupE es) = hashParens $ sep $ punctuate comma $ map ppr es
 -- Nesting in Cond is to avoid potential problems in do statments
 pprExp i (CondE guard true false)
  = parensIf (i > noPrec) $ sep [text "if"   <+> ppr guard,
@@ -172,10 +172,15 @@ pprLit i (DoublePrimL x) = parensIf (i > noPrec && x < 0)
                                     (double (fromRational x) <> text "##")
 pprLit i (IntegerL x)    = parensIf (i > noPrec && x < 0) (integer x)
 pprLit _ (CharL c)       = text (show c)
-pprLit _ (StringL s)     = text (show s)
-pprLit _ (StringPrimL s) = text (show s) <> char '#'
+pprLit _ (StringL s)     = pprString s
+pprLit _ (StringPrimL s) = pprString s <> char '#'
 pprLit i (RationalL rat) = parensIf (i > noPrec) $ rational rat
 
+pprString :: String -> Doc
+-- Print newlines as newlines with Haskell string escape notation, 
+-- not as '\n'.  For other non-printables use regular escape notation.
+pprString s = vcat (map text (showMultiLineString s))
+
 ------------------------------
 instance Ppr Pat where
     ppr = pprPat noPrec
@@ -184,8 +189,14 @@ pprPat :: Precedence -> Pat -> Doc
 pprPat i (LitP l)     = pprLit i l
 pprPat _ (VarP v)     = pprName' Applied v
 pprPat _ (TupP ps)    = parens $ sep $ punctuate comma $ map ppr ps
+pprPat _ (UnboxedTupP ps) = hashParens $ sep $ punctuate comma $ map ppr ps
 pprPat i (ConP s ps)  = parensIf (i >= appPrec) $ pprName' Applied s
                                               <+> sep (map (pprPat appPrec) ps)
+pprPat _ (ParensP p)  = parens $ pprPat noPrec p
+pprPat i (UInfixP p1 n p2)
+                      = parensIf (i > unopPrec) (pprPat unopPrec p1 <+>
+                                                 pprName' Infix n   <+>
+                                                 pprPat unopPrec p2)
 pprPat i (InfixP p1 n p2)
                       = parensIf (i >= opPrec) (pprPat opPrec p1 <+>
                                                 pprName' Infix n <+>
@@ -366,6 +377,7 @@ pprStrictType :: (Strict, Type) -> Doc
 -- Prints with parens if not already atomic
 pprStrictType (IsStrict, t) = char '!' <> pprParendType t
 pprStrictType (NotStrict, t) = pprParendType t
+pprStrictType (Unpacked, t) = text "{-# UNPACK #-} !" <> pprParendType t
 
 ------------------------------
 pprParendType :: Type -> Doc
@@ -373,6 +385,7 @@ pprParendType (VarT v)   = ppr v
 pprParendType (ConT c)   = ppr c
 pprParendType (TupleT 0) = text "()"
 pprParendType (TupleT n) = parens (hcat (replicate (n-1) comma))
+pprParendType (UnboxedTupleT n) = hashParens $ hcat $ replicate (n-1) comma
 pprParendType ArrowT     = parens (text "->")
 pprParendType ListT      = text "[]"
 pprParendType other      = parens (ppr other)
@@ -447,3 +460,6 @@ where_clause ds = nest nestDepth $ text "where" <+> vcat (map (ppr_dec False) ds
 showtextl :: Show a => a -> Doc
 showtextl = text . map toLower . show
 
+hashParens :: Doc -> Doc
+hashParens d = text "(# " <> d <> text " #)"
+
