@@ -46,9 +46,9 @@ void sendMessage(Capability *from_cap, Capability *to_cap, Message *msg)
     if (to_cap->running_task == NULL) {
 	to_cap->running_task = myTask(); 
             // precond for releaseCapability_()
-	releaseCapability_(to_cap,rtsFalse);
+        releaseCapability_(to_cap,rtsFalse);
     } else {
-        contextSwitchCapability(to_cap);
+        interruptCapability(to_cap);
     }
 
     RELEASE_LOCK(&to_cap->lock);
