@@ -60,10 +60,9 @@ char *EventDesc[] = {
   [EVENT_STOP_THREAD]         = "Stop thread",
   [EVENT_THREAD_RUNNABLE]     = "Thread runnable",
   [EVENT_MIGRATE_THREAD]      = "Migrate thread",
-  [EVENT_RUN_SPARK]           = "Run spark",
-  [EVENT_STEAL_SPARK]         = "Steal spark",
   [EVENT_SHUTDOWN]            = "Shutdown",
   [EVENT_THREAD_WAKEUP]       = "Wakeup thread",
+  [EVENT_THREAD_LABEL]        = "Thread label",
   [EVENT_GC_START]            = "Starting GC",
   [EVENT_GC_END]              = "Finished GC",
   [EVENT_REQUEST_SEQ_GC]      = "Request sequential GC",
@@ -84,7 +83,16 @@ char *EventDesc[] = {
   [EVENT_PROGRAM_ARGS]        = "Program arguments",
   [EVENT_PROGRAM_ENV]         = "Program environment variables",
   [EVENT_OSPROCESS_PID]       = "Process ID",
-  [EVENT_OSPROCESS_PPID]      = "Parent process ID"
+  [EVENT_OSPROCESS_PPID]      = "Parent process ID",
+  [EVENT_WALL_CLOCK_TIME]     = "Wall clock time",
+  [EVENT_SPARK_COUNTERS]      = "Spark counters",
+  [EVENT_SPARK_CREATE]        = "Spark create",
+  [EVENT_SPARK_DUD]           = "Spark dud",
+  [EVENT_SPARK_OVERFLOW]      = "Spark overflow",
+  [EVENT_SPARK_RUN]           = "Spark run",
+  [EVENT_SPARK_STEAL]         = "Spark steal",
+  [EVENT_SPARK_FIZZLE]        = "Spark fizzle",
+  [EVENT_SPARK_GC]            = "Spark GC",
 };
 
 // Event type. 
@@ -95,7 +103,7 @@ typedef struct _EventType {
   char *desc;     // Description
 } EventType;
 
-EventType eventTypes[NUM_EVENT_TAGS];
+EventType eventTypes[NUM_GHC_EVENT_TAGS];
 
 static void initEventsBuf(EventsBuf* eb, StgWord64 size, EventCapNo capno);
 static void resetEventsBuf(EventsBuf* eb);
@@ -141,7 +149,7 @@ static inline void postBuf(EventsBuf *eb, StgWord8 *buf, nat size)
 }
 
 static inline StgWord64 time_ns(void)
-{ return stat_getElapsedTime() * (1000000000LL/TICKS_PER_SECOND); }
+{ return TimeToNS(stat_getElapsedTime()); }
 
 static inline void postEventTypeNum(EventsBuf *eb, EventTypeNum etNum)
 { postWord16(eb, etNum); }
@@ -188,26 +196,41 @@ initEventLogging(void)
 {
     StgWord8 t, c;
     nat n_caps;
+    char *prog;
 
-    event_log_filename = stgMallocBytes(strlen(prog_name)
+    prog = stgMallocBytes(strlen(prog_name) + 1, "initEventLogging");
+    strcpy(prog, prog_name);
+#ifdef mingw32_HOST_OS
+    // on Windows, drop the .exe suffix if there is one
+    {
+        char *suff;
+        suff = strrchr(prog,'.');
+        if (suff != NULL && !strcmp(suff,".exe")) {
+            *suff = '\0';
+        }
+    }
+#endif
+
+    event_log_filename = stgMallocBytes(strlen(prog)
                                         + 10 /* .%d */
                                         + 10 /* .eventlog */,
                                         "initEventLogging");
 
-    if (sizeof(EventDesc) / sizeof(char*) != NUM_EVENT_TAGS) {
+    if (sizeof(EventDesc) / sizeof(char*) != NUM_GHC_EVENT_TAGS) {
         barf("EventDesc array has the wrong number of elements");
     }
 
     if (event_log_pid == -1) { // #4512
         // Single process
-        sprintf(event_log_filename, "%s.eventlog", prog_name);
+        sprintf(event_log_filename, "%s.eventlog", prog);
         event_log_pid = getpid();
     } else {
         // Forked process, eventlog already started by the parent
         // before fork
         event_log_pid = getpid();
-        sprintf(event_log_filename, "%s.%d.eventlog", prog_name, event_log_pid);
+        sprintf(event_log_filename, "%s.%d.eventlog", prog, event_log_pid);
     }
+    stgFree(prog);
 
     /* Open event log file for writing. */
     if ((event_log_file = fopen(event_log_filename, "wb")) == NULL) {
@@ -231,12 +254,8 @@ initEventLogging(void)
 #else
     n_caps = 1;
 #endif
-    capEventBuf = stgMallocBytes(n_caps * sizeof(EventsBuf),"initEventLogging");
+    moreCapEventBufs(0,n_caps);
 
-    for (c = 0; c < n_caps; ++c) {
-        // Init buffer for events.
-        initEventsBuf(&capEventBuf[c], EVENT_LOG_SIZE, c);
-    }
     initEventsBuf(&eventBuf, EVENT_LOG_SIZE, (EventCapNo)(-1));
 
     // Write in buffer: the header begin marker.
@@ -244,7 +263,7 @@ initEventLogging(void)
 
     // Mark beginning of event types in the header.
     postInt32(&eventBuf, EVENT_HET_BEGIN);
-    for (t = 0; t < NUM_EVENT_TAGS; ++t) {
+    for (t = 0; t < NUM_GHC_EVENT_TAGS; ++t) {
 
         eventTypes[t].etNum = t;
         eventTypes[t].desc = EventDesc[t];
@@ -253,13 +272,11 @@ initEventLogging(void)
         case EVENT_CREATE_THREAD:   // (cap, thread)
         case EVENT_RUN_THREAD:      // (cap, thread)
         case EVENT_THREAD_RUNNABLE: // (cap, thread)
-        case EVENT_RUN_SPARK:       // (cap, thread)
         case EVENT_CREATE_SPARK_THREAD: // (cap, spark_thread)
             eventTypes[t].size = sizeof(EventThreadID);
             break;
 
         case EVENT_MIGRATE_THREAD:  // (cap, thread, new_cap)
-        case EVENT_STEAL_SPARK:     // (cap, thread, victim_cap)
         case EVENT_THREAD_WAKEUP:   // (cap, thread, other_cap)
             eventTypes[t].size =
                 sizeof(EventThreadID) + sizeof(EventCapNo);
@@ -295,6 +312,16 @@ initEventLogging(void)
                 sizeof(EventCapsetID) + sizeof(StgWord32);
             break;
 
+        case EVENT_WALL_CLOCK_TIME: // (capset, unix_epoch_seconds, nanoseconds)
+            eventTypes[t].size =
+                sizeof(EventCapsetID) + sizeof(StgWord64) + sizeof(StgWord32);
+            break;
+
+        case EVENT_SPARK_STEAL:     // (cap, victim_cap)
+            eventTypes[t].size =
+                sizeof(EventCapNo);
+            break;
+
         case EVENT_SHUTDOWN:        // (cap)
         case EVENT_REQUEST_SEQ_GC:  // (cap)
         case EVENT_REQUEST_PAR_GC:  // (cap)
@@ -303,6 +330,12 @@ initEventLogging(void)
         case EVENT_GC_IDLE:
         case EVENT_GC_WORK:
         case EVENT_GC_DONE:
+        case EVENT_SPARK_CREATE:    // (cap)
+        case EVENT_SPARK_DUD:       // (cap)
+        case EVENT_SPARK_OVERFLOW:  // (cap)
+        case EVENT_SPARK_RUN:       // (cap)
+        case EVENT_SPARK_FIZZLE:    // (cap)
+        case EVENT_SPARK_GC:        // (cap)
             eventTypes[t].size = 0;
             break;
 
@@ -311,9 +344,14 @@ initEventLogging(void)
         case EVENT_RTS_IDENTIFIER:   // (capset, str)
         case EVENT_PROGRAM_ARGS:     // (capset, strvec)
         case EVENT_PROGRAM_ENV:      // (capset, strvec)
+        case EVENT_THREAD_LABEL:     // (thread, str)
             eventTypes[t].size = 0xffff;
             break;
 
+        case EVENT_SPARK_COUNTERS:   // (cap, 7*counter)
+            eventTypes[t].size = 7 * sizeof(StgWord64);
+            break;
+
         case EVENT_BLOCK_MARKER:
             eventTypes[t].size = sizeof(StgWord32) + sizeof(EventTimestamp) + 
                 sizeof(EventCapNo);
@@ -375,7 +413,26 @@ endEventLogging(void)
     }
 }
 
-void 
+void
+moreCapEventBufs (nat from, nat to)
+{
+    nat c;
+
+    if (from > 0) {
+        capEventBuf = stgReallocBytes(capEventBuf, to * sizeof(EventsBuf),
+                                      "moreCapEventBufs");
+    } else {
+        capEventBuf = stgMallocBytes(to * sizeof(EventsBuf),
+                                     "moreCapEventBufs");
+    }
+
+    for (c = from; c < to; ++c) {
+        initEventsBuf(&capEventBuf[c], EVENT_LOG_SIZE, c);
+    }
+}
+
+
+void
 freeEventLogging(void)
 {
     StgWord8 c;
@@ -435,7 +492,6 @@ postSchedEvent (Capability *cap,
     case EVENT_CREATE_THREAD:   // (cap, thread)
     case EVENT_RUN_THREAD:      // (cap, thread)
     case EVENT_THREAD_RUNNABLE: // (cap, thread)
-    case EVENT_RUN_SPARK:       // (cap, thread)
     {
         postThreadID(eb,thread);
         break;
@@ -448,7 +504,6 @@ postSchedEvent (Capability *cap,
     }
 
     case EVENT_MIGRATE_THREAD:  // (cap, thread, new_cap)
-    case EVENT_STEAL_SPARK:     // (cap, thread, victim_cap)
     case EVENT_THREAD_WAKEUP:   // (cap, thread, other_cap)
     {
         postThreadID(eb,thread);
@@ -465,22 +520,86 @@ postSchedEvent (Capability *cap,
     }
 
     case EVENT_SHUTDOWN:        // (cap)
-    case EVENT_REQUEST_SEQ_GC:  // (cap)
-    case EVENT_REQUEST_PAR_GC:  // (cap)
-    case EVENT_GC_START:        // (cap)
-    case EVENT_GC_END:          // (cap)
     {
         break;
     }
 
     default:
-        barf("postEvent: unknown event tag %d", tag);
+        barf("postSchedEvent: unknown event tag %d", tag);
     }
 }
 
-void postCapsetModifyEvent (EventTypeNum tag,
-                            EventCapsetID capset,
-                            StgWord32 other)
+void
+postSparkEvent (Capability *cap,
+                EventTypeNum tag,
+                StgWord info1)
+{
+    EventsBuf *eb;
+
+    eb = &capEventBuf[cap->no];
+
+    if (!hasRoomForEvent(eb, tag)) {
+        // Flush event buffer to make room for new event.
+        printAndClearEventBuf(eb);
+    }
+
+    postEventHeader(eb, tag);
+
+    switch (tag) {
+    case EVENT_CREATE_SPARK_THREAD: // (cap, spark_thread)
+    {
+        postThreadID(eb,info1 /* spark_thread */);
+        break;
+    }
+
+    case EVENT_SPARK_STEAL:         // (cap, victim_cap)
+    {
+        postCapNo(eb,info1 /* victim_cap */);
+        break;
+   }
+
+    case EVENT_SPARK_CREATE:        // (cap)
+    case EVENT_SPARK_DUD:           // (cap)
+    case EVENT_SPARK_OVERFLOW:      // (cap)
+    case EVENT_SPARK_RUN:           // (cap)
+    case EVENT_SPARK_FIZZLE:        // (cap)
+    case EVENT_SPARK_GC:            // (cap)
+    {
+        break;
+    }
+
+    default:
+        barf("postSparkEvent: unknown event tag %d", tag);
+    }
+}
+
+void
+postSparkCountersEvent (Capability *cap, 
+                        SparkCounters counters,
+                        StgWord remaining)
+{
+    EventsBuf *eb;
+
+    eb = &capEventBuf[cap->no];
+
+    if (!hasRoomForEvent(eb, EVENT_SPARK_COUNTERS)) {
+        // Flush event buffer to make room for new event.
+        printAndClearEventBuf(eb);
+    }
+    
+    postEventHeader(eb, EVENT_SPARK_COUNTERS);
+    postWord64(eb,counters.created);
+    postWord64(eb,counters.dud);
+    postWord64(eb,counters.overflowed);
+    postWord64(eb,counters.converted);
+    postWord64(eb,counters.gcd);
+    postWord64(eb,counters.fizzled);
+    postWord64(eb,remaining);
+}
+
+void postCapsetEvent (EventTypeNum tag,
+                      EventCapsetID capset,
+                      StgWord info)
 {
     ACQUIRE_LOCK(&eventBufMutex);
 
@@ -495,7 +614,7 @@ void postCapsetModifyEvent (EventTypeNum tag,
     switch (tag) {
     case EVENT_CAPSET_CREATE:   // (capset, capset_type)
     {
-        postCapsetType(&eventBuf, other /* capset_type */);
+        postCapsetType(&eventBuf, info /* capset_type */);
         break;
     }
 
@@ -507,17 +626,17 @@ void postCapsetModifyEvent (EventTypeNum tag,
     case EVENT_CAPSET_ASSIGN_CAP:  // (capset, capno)
     case EVENT_CAPSET_REMOVE_CAP:  // (capset, capno)
     {
-        postCapNo(&eventBuf, other /* capno */);
+        postCapNo(&eventBuf, info /* capno */);
         break;
     }
     case EVENT_OSPROCESS_PID:   // (capset, pid)
     case EVENT_OSPROCESS_PPID:  // (capset, parent_pid)
     {
-        postWord32(&eventBuf, other);
+        postWord32(&eventBuf, info);
         break;
     }
     default:
-        barf("postCapsetModifyEvent: unknown event tag %d", tag);
+        barf("postCapsetEvent: unknown event tag %d", tag);
     }
 
     RELEASE_LOCK(&eventBufMutex);
@@ -587,6 +706,52 @@ void postCapsetVecEvent (EventTypeNum tag,
     RELEASE_LOCK(&eventBufMutex);
 }
 
+void postWallClockTime (EventCapsetID capset)
+{
+    StgWord64 ts;
+    StgWord64 sec;
+    StgWord32 nsec;
+
+    ACQUIRE_LOCK(&eventBufMutex);
+    
+    /* The EVENT_WALL_CLOCK_TIME event is intended to allow programs
+       reading the eventlog to match up the event timestamps with wall
+       clock time. The normal event timestamps measure time since the
+       start of the program. To align eventlogs from concurrent
+       processes we need to be able to match up the timestamps. One way
+       to do this is if we know how the timestamps and wall clock time
+       match up (and of course if both processes have sufficiently
+       synchronised clocks).
+
+       So we want to make sure that the timestamp that we generate for
+       this event matches up very closely with the wall clock time.
+       Unfortunately we currently have to use two different APIs to get
+       the elapsed time vs the wall clock time. So to minimise the
+       difference we just call them very close together.
+     */
+    
+    getUnixEpochTime(&sec, &nsec);  /* Get the wall clock time */
+    ts = time_ns();                 /* Get the eventlog timestamp */
+
+    if (!hasRoomForEvent(&eventBuf, EVENT_WALL_CLOCK_TIME)) {
+        // Flush event buffer to make room for new event.
+        printAndClearEventBuf(&eventBuf);
+    }
+
+    /* Normally we'd call postEventHeader(), but that generates its own
+       timestamp, so we go one level lower so we can write out the
+       timestamp we already generated above. */
+    postEventTypeNum(&eventBuf, EVENT_WALL_CLOCK_TIME);
+    postWord64(&eventBuf, ts);
+    
+    /* EVENT_WALL_CLOCK_TIME (capset, unix_epoch_seconds, nanoseconds) */
+    postCapsetID(&eventBuf, capset);
+    postWord64(&eventBuf, sec);
+    postWord32(&eventBuf, nsec);
+
+    RELEASE_LOCK(&eventBufMutex);
+}
+
 void
 postEvent (Capability *cap, EventTypeNum tag)
 {
@@ -658,6 +823,31 @@ void postEventStartup(EventCapNo n_caps)
     RELEASE_LOCK(&eventBufMutex);
 }
 
+void postThreadLabel(Capability    *cap,
+                     EventThreadID  id,
+                     char          *label)
+{
+    EventsBuf *eb;
+    int strsize = strlen(label);
+    int size = strsize + sizeof(EventCapsetID);
+
+    eb = &capEventBuf[cap->no];
+
+    if (!hasRoomForVariableEvent(eb, size)){
+        printAndClearEventBuf(eb);
+
+        if (!hasRoomForVariableEvent(eb, size)){
+            // Event size exceeds buffer size, bail out:
+            return;
+        }
+    }
+
+    postEventHeader(eb, EVENT_THREAD_LABEL);
+    postPayloadSize(eb, size);
+    postThreadID(eb, id);
+    postBuf(eb, (StgWord8*) label, strsize);
+}
+
 void closeBlockMarker (EventsBuf *ebuf)
 {
     StgInt8* save_pos;
